"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var wavFileInfo = require("wav-file-info");
var fsExtra = require("fs-extra");
var cli_1 = require("../../cli");
var ssh_1 = require("../../session/ssh");
var promises_1 = require("../../utils/promises");
var stats_1 = require("../../utils/stats");
var functions_1 = require("../../utils/functions");
var chalk_1 = require("chalk");
exports.command = 'asr <wavs_filepath> <assistant_filepaths...>';
exports.desc = 'Test ASR performances with a given wav file';
exports.builder = {
    passes: {
        demandOption: false,
        describe: 'Number of times to play the waves on each assistant',
        type: 'number',
        default: 1,
    },
    sigmaPrecision: {
        demandOption: false,
        describe: 'Sigma precision used during the bisection. It will be used to divide the STD found during benchmarks.',
        type: 'number',
        default: 1,
        alias: 'sigma',
    },
    defaultMaxActive: {
        demandOption: false,
        describe: 'Default max active value used during the bisection.',
        type: 'number',
        default: 2000,
        alias: 'max',
    },
    defaultMinActive: {
        demandOption: false,
        describe: 'Default min active value used during the bisection.',
        type: 'number',
        default: 200,
        alias: 'min',
    },
    stepSize: {
        demandOption: false,
        describe: 'Step size used in the bisection.',
        type: 'number',
        default: 200,
        alias: 'step',
    },
};
var BisectionResult = (function () {
    function BisectionResult() {
        this.minActive = 0;
        this.maxActive = 0;
        this.asrResults = [];
    }
    return BisectionResult;
}());
exports.BisectionResult = BisectionResult;
var ASRResult = (function () {
    function ASRResult(assistant) {
        this.rtfs = [];
        this.max = 0;
        this.min = 0;
        this.mean = 0;
        this.variance = 0;
        this.std = 0;
        this.assistant = assistant;
    }
    ASRResult.prototype.processRTFs = function () {
        var results = functions_1.flatMap(this.rtfs, function (value) { return value; });
        this.max = Math.max.apply(Math, results);
        this.min = Math.min.apply(Math, results);
        this.mean = stats_1.Stats.mean(results);
        this.variance = stats_1.Stats.variance(results);
        this.std = stats_1.Stats.standardDeviation(results);
    };
    return ASRResult;
}());
exports.ASRResult = ASRResult;
exports.handler = function (argv) { return __awaiter(_this, void 0, void 0, function () {
    var _this = this;
    var assistantPaths, wavFilePath, passes, sigmaPrecision, defaultMaxActive, defaultMinActive, defaultStepSize, benchmark, assistantResults, wavFiles, _i, assistantPaths_1, assistantPath, assistantName, currentASRResult, index, rtfsResults, _loop_1, _a, assistantPaths_2, assistantPath, e_1;
    return __generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                process.on('SIGINT', function () { return __awaiter(_this, void 0, void 0, function () {
                    return __generator(this, function (_a) {
                        switch (_a.label) {
                            case 0:
                                cli_1.cli.stream.stopAndClear();
                                cli_1.cli.stream.loading('Resetting values on device before exiting');
                                return [4, benchmark.reset().catch(function (e) { return cli_1.cli.stream.error('Resetting services failed: ' + e); })];
                            case 1:
                                _a.sent();
                                cli_1.cli.stream.success();
                                process.exit();
                                return [2];
                        }
                    });
                }); });
                cli_1.cli.stream.println('Testing ASR');
                assistantPaths = argv['assistant_filepaths'];
                wavFilePath = argv['wavs_filepath'];
                passes = JSON.parse(JSON.stringify(argv)).passes;
                sigmaPrecision = JSON.parse(JSON.stringify(argv)).sigmaPrecision;
                defaultMaxActive = JSON.parse(JSON.stringify(argv)).defaultMaxActive;
                defaultMinActive = JSON.parse(JSON.stringify(argv)).defaultMinActive;
                defaultStepSize = JSON.parse(JSON.stringify(argv)).stepSize;
                benchmark = new ASRBenchmark(cli_1.cli);
                assistantResults = [];
                _b.label = 1;
            case 1:
                _b.trys.push([1, 18, , 19]);
                return [4, benchmark.getWAVFileFromPathAndCheckValidity(wavFilePath)];
            case 2:
                wavFiles = _b.sent();
                if (wavFiles.length === 0) {
                    throw new Error("No wavs found in the folder, wrong path ?");
                }
                return [4, benchmark.connect()];
            case 3:
                _b.sent();
                return [4, benchmark.setupServicesOnDevice()];
            case 4:
                _b.sent();
                return [4, benchmark.deployWAVFolder(wavFilePath)];
            case 5:
                _b.sent();
                _i = 0, assistantPaths_1 = assistantPaths;
                _b.label = 6;
            case 6:
                if (!(_i < assistantPaths_1.length)) return [3, 13];
                assistantPath = assistantPaths_1[_i];
                assistantName = path.basename(assistantPath);
                currentASRResult = new ASRResult(assistantName);
                return [4, benchmark.deployAssistant(assistantPath)];
            case 7:
                _b.sent();
                index = 0;
                _b.label = 8;
            case 8:
                if (!(index < passes)) return [3, 11];
                return [4, benchmark.launchBenchmark(assistantName, wavFiles)];
            case 9:
                rtfsResults = _b.sent();
                currentASRResult.rtfs.push(rtfsResults);
                _b.label = 10;
            case 10:
                index += 1;
                return [3, 8];
            case 11:
                assistantResults.push(currentASRResult);
                _b.label = 12;
            case 12:
                _i++;
                return [3, 6];
            case 13:
                assistantResults.forEach(function (asrResult) {
                    asrResult.processRTFs();
                    cli_1.cli.stream.println("\nAssistant " + asrResult.assistant);
                    cli_1.cli.stream.println("RTFs iterations:");
                    asrResult.rtfs.forEach(function (rtfsResultsByIteration, index) {
                        cli_1.cli.stream.println(index + 1 + " \t " + rtfsResultsByIteration);
                    });
                    cli_1.cli.stream.println("Max \t " + asrResult.max);
                    cli_1.cli.stream.println("Min \t " + asrResult.min);
                    cli_1.cli.stream.println("Mean \t " + asrResult.mean.toFixed(4));
                    cli_1.cli.stream.println("Var \t " + asrResult.variance.toFixed(4));
                    cli_1.cli.stream.println("STD \t " + asrResult.std.toFixed(4));
                });
                _loop_1 = function (assistantPath) {
                    var assistantName, assistantResult;
                    return __generator(this, function (_a) {
                        switch (_a.label) {
                            case 0:
                                cli_1.cli.stream.print('\n');
                                assistantName = path.basename(assistantPath);
                                assistantResult = assistantResults.filter(function (asrResult) { return asrResult.assistant = assistantName; })[0];
                                return [4, benchmark.deployAssistant(assistantPath)];
                            case 1:
                                _a.sent();
                                return [4, benchmark.bisection(assistantResult.assistant, wavFiles, defaultMinActive, defaultMaxActive, assistantResult.std / sigmaPrecision, defaultStepSize, passes)];
                            case 2:
                                _a.sent();
                                return [2];
                        }
                    });
                };
                _a = 0, assistantPaths_2 = assistantPaths;
                _b.label = 14;
            case 14:
                if (!(_a < assistantPaths_2.length)) return [3, 17];
                assistantPath = assistantPaths_2[_a];
                return [5, _loop_1(assistantPath)];
            case 15:
                _b.sent();
                _b.label = 16;
            case 16:
                _a++;
                return [3, 14];
            case 17: return [3, 19];
            case 18:
                e_1 = _b.sent();
                cli_1.cli.stream.error(e_1);
                benchmark.reset().catch(function (e) { return cli_1.cli.stream.error('Resetting services failed: ' + e); });
                process.exit();
                return [3, 19];
            case 19:
                cli_1.cli.stream.success('Finished');
                benchmark.reset().catch(function (e) { return cli_1.cli.stream.error('Resetting services failed: ' + e); });
                process.exit();
                return [2];
        }
    });
}); };
var ASRBenchmark = (function () {
    function ASRBenchmark(cli) {
        var _this = this;
        this.remoteAudioPath = '/tmp/audio';
        this.connect = function () { return __awaiter(_this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.ssh.connect().then(function (r) { return _this.journalctlLogListener(); })];
                    case 1: return [2, _a.sent()];
                }
            });
        }); };
        this.reset = function () { return __awaiter(_this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2, this.ssh.resetASRService()
                        .then(function (s) { return _this.ssh.resetAudioServerService(); })
                        .then(function (s) { return _this.ssh.removeFolder(_this.remoteAudioPath); })
                        .then(function (s) { return _this.ssh.disconnect(); })];
            });
        }); };
        this.setupServicesOnDevice = function () { return __awaiter(_this, void 0, void 0, function () {
            var _this = this;
            var mosquittoClients, netcat;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.cli.stream.loading('Editing Audio Server service');
                        return [4, this.ssh.editAudioServerService()
                                .catch(function (e) {
                                _this.cli.stream.error("Editing Audio Server service failed: " + e);
                                _this.ssh.disconnect();
                                process.exit();
                            })];
                    case 1:
                        _a.sent();
                        this.cli.stream.success();
                        mosquittoClients = 'mosquitto-clients';
                        this.cli.stream.loading("Checking if " + mosquittoClients + " is installed");
                        return [4, this.ssh.dpkgCheckIfPackageIsInstalled(mosquittoClients)
                                .then(function (result) { return result === true ? 'success' : _this.ssh.aptGet(mosquittoClients, function () { }); })
                                .catch(function (e) {
                                _this.cli.stream.error("Failed installing " + mosquittoClients + " package, can't run the benchmark without it");
                                _this.ssh.disconnect();
                                process.exit();
                            })];
                    case 2:
                        _a.sent();
                        this.cli.stream.success();
                        netcat = 'netcat';
                        this.cli.stream.loading("Checking if " + netcat + " is installed");
                        return [4, this.ssh.isNetcatInstalled()
                                .then(function (result) { return result === true ? 'success' : _this.ssh.aptGet(netcat, function () { }); })
                                .catch(function (e) {
                                _this.cli.stream.error("Failed installing " + netcat + " package, can't run the benchmark without it");
                                _this.ssh.disconnect();
                                process.exit();
                            })];
                    case 3:
                        _a.sent();
                        this.cli.stream.success();
                        return [2, 'success'];
                }
            });
        }); };
        this.deployAssistant = function (assistantPath) { return __awaiter(_this, void 0, void 0, function () {
            var currentAssistant;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        currentAssistant = path.basename(assistantPath);
                        this.cli.stream.loading("Deploying assistant " + currentAssistant);
                        return [4, this.ssh.installAssistant(assistantPath)
                                .catch(function (e) { throw new Error('Installing assistant: ' + e.message); })];
                    case 1:
                        _a.sent();
                        this.cli.stream.success();
                        return [2];
                }
            });
        }); };
        this.deployWAVFolder = function (wavFolderPath) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.cli.stream.loading('Deploying wavs');
                        return [4, this.ssh.putDirectory(wavFolderPath, this.remoteAudioPath)
                                .catch(function (e) {
                                throw new Error("Couldn't copy wav on the device: " + e);
                            })];
                    case 1:
                        _a.sent();
                        this.cli.stream.success();
                        return [2];
                }
            });
        }); };
        this.bisection = function (assistantName, wavFiles, minActive, maxActive, stdReference, stepSize, benchmarkPasses) { return __awaiter(_this, void 0, void 0, function () {
            var currentMaxActive, currentMinActive, currentMean, upperBoundSTD, lowerBoundSTD, asrResult, index, rtfsResults;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        currentMaxActive = maxActive;
                        currentMinActive = minActive;
                        currentMean = 0;
                        upperBoundSTD = Number((1 + stdReference).toFixed(4));
                        lowerBoundSTD = Number((1 - stdReference).toFixed(4));
                        this.cli.stream.println("Target: " + lowerBoundSTD + " < mean rtf < " + upperBoundSTD);
                        _a.label = 1;
                    case 1:
                        if (!(currentMinActive < currentMaxActive)) return [3, 7];
                        this.cli.stream.print('\n');
                        this.cli.stream.println(chalk_1.default.green('New bisection loop'));
                        this.cli.stream.println("minActive: " + currentMinActive + " maxActive: " + currentMaxActive);
                        asrResult = new ASRResult(assistantName);
                        currentMean = Math.round((currentMinActive + currentMaxActive) / 2);
                        this.cli.stream.println("Taking mean of " + currentMean);
                        return [4, this.setupASRService(currentMean, currentMean)];
                    case 2:
                        _a.sent();
                        index = 0;
                        _a.label = 3;
                    case 3:
                        if (!(index < benchmarkPasses)) return [3, 6];
                        return [4, this.launchBenchmark(assistantName, wavFiles)];
                    case 4:
                        rtfsResults = _a.sent();
                        asrResult.rtfs.push(rtfsResults);
                        _a.label = 5;
                    case 5:
                        index += 1;
                        return [3, 3];
                    case 6:
                        asrResult.processRTFs();
                        this.cli.stream.println("RTFS mean " + asrResult.mean.toFixed(4));
                        if (asrResult.mean > upperBoundSTD) {
                            this.cli.stream.println(chalk_1.default.red('> ' + upperBoundSTD));
                            currentMaxActive = currentMean;
                        }
                        else if (asrResult.mean < lowerBoundSTD) {
                            this.cli.stream.println(chalk_1.default.red('< ' + lowerBoundSTD));
                            currentMinActive += stepSize;
                        }
                        else {
                            return [3, 7];
                        }
                        return [3, 1];
                    case 7:
                        this.cli.stream.println(chalk_1.default.green('Found possible match'));
                        this.cli.stream.println("mean active used : " + currentMean);
                        this.cli.stream.println("minActive : " + currentMinActive);
                        this.cli.stream.println("maxActive : " + currentMaxActive);
                        return [2];
                }
            });
        }); };
        this.getWAVFileFromPathAndCheckValidity = function (wavFilePath) { return __awaiter(_this, void 0, void 0, function () {
            var fileNames, wavFiles, _i, fileNames_1, name, filePath, duration, e_2;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 5, , 6]);
                        fileNames = fsExtra.readdirSync(wavFilePath);
                        wavFiles = [];
                        _i = 0, fileNames_1 = fileNames;
                        _a.label = 1;
                    case 1:
                        if (!(_i < fileNames_1.length)) return [3, 4];
                        name = fileNames_1[_i];
                        filePath = path.join(wavFilePath, name);
                        return [4, this.getWAVDuration(filePath)];
                    case 2:
                        duration = _a.sent();
                        wavFiles.push({ name: name, duration: duration, path: filePath });
                        _a.label = 3;
                    case 3:
                        _i++;
                        return [3, 1];
                    case 4: return [2, wavFiles];
                    case 5:
                        e_2 = _a.sent();
                        throw new Error("Error getting the wav " + wavFilePath + " with error: " + e_2);
                    case 6: return [2];
                }
            });
        }); };
        this.setupASRService = function (maxActive, minActive) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.cli.stream.loading("Setting ASR service with min: " + minActive + " max: " + maxActive);
                        return [4, this.ssh.editASRService(maxActive, minActive)];
                    case 1:
                        _a.sent();
                        this.cli.stream.success();
                        this.cli.stream.loading('Relaunching ASR service');
                        return [4, promises_1.timeout(15000)];
                    case 2:
                        _a.sent();
                        this.cli.stream.success();
                        return [2];
                }
            });
        }); };
        this.launchBenchmark = function (assistantName, wavFiles) { return __awaiter(_this, void 0, void 0, function () {
            var _this = this;
            var rtfsResults, _loop_2, this_1, _i, wavFiles_1, file;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.cli.stream.println(chalk_1.default.green("Launching benchmark on " + assistantName));
                        rtfsResults = [];
                        _loop_2 = function (file) {
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4, this_1.playRemoteWAV(file.duration, this_1.remoteAudioPath + "/" + file.name)
                                            .then(function (rtf) {
                                            if (rtf > 0) {
                                                rtfsResults.push(rtf);
                                                _this.cli.stream.print(chalk_1.default.blue(file.name) + " " + rtf + " ");
                                            }
                                        })];
                                    case 1:
                                        _a.sent();
                                        return [4, this_1.ssh.killNCServer()];
                                    case 2:
                                        _a.sent();
                                        return [2];
                                }
                            });
                        };
                        this_1 = this;
                        _i = 0, wavFiles_1 = wavFiles;
                        _a.label = 1;
                    case 1:
                        if (!(_i < wavFiles_1.length)) return [3, 4];
                        file = wavFiles_1[_i];
                        return [5, _loop_2(file)];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3:
                        _i++;
                        return [3, 1];
                    case 4:
                        this.cli.stream.print('\n');
                        return [2, rtfsResults];
                }
            });
        }); };
        this.playRemoteWAV = function (wavDuration, wavRemotePath) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.ssh.publishMQTTStartListening()];
                    case 1:
                        _a.sent();
                        this.ssh.playWAVtoASR(wavRemotePath)
                            .catch(function (e) { throw new Error('Error playing wav : ' + e.message); });
                        this.rtfResultPromise = new promises_1.Deferred((wavDuration * 1000) + 5000);
                        return [4, this.rtfResultPromise.promise
                                .catch(function (e) { throw new Error("Playing wav didn't trigger a new RTF measure : " + e); })];
                    case 2: return [2, _a.sent()];
                }
            });
        }); };
        this.journalctlLogListener = function () { return __awaiter(_this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                this.ssh.journalctlLog('snips-asr', function (response) {
                    if (response.includes('endpoint detected')) {
                        var rtf = _this.extractRTFfromEndpointingLog(response);
                        if (rtf === undefined) {
                            _this.rtfResultPromise.reject();
                            return;
                        }
                        _this.rtfResultPromise.resolve(rtf);
                    }
                });
                return [2];
            });
        }); };
        this.getWAVDuration = function (wavPath) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2, new Promise(function (resolve, reject) {
                        wavFileInfo.infoByFilename(wavPath, function (err, info) {
                            if (err) {
                                reject("Couldn't open the wav file: " + err.message);
                            }
                            var header = info['header'];
                            if (header['num_channels'] !== 1) {
                                reject("Number of channels: " + header['num_channels'] + ", expected 1");
                            }
                            if (header['sample_rate'] !== 16000) {
                                reject("Sample rate: " + header['sample_rate'] + ", expected 16000");
                            }
                            if (header['bits_per_sample'] !== 16) {
                                reject("Bits per sample: " + header['bits_per_sample'] + ", expected 16");
                            }
                            resolve(info['duration']);
                        });
                    })];
            });
        }); };
        this.ssh = new ssh_1.SSHService();
        this.cli = cli;
    }
    ASRBenchmark.prototype.extractRTFfromEndpointingLog = function (log) {
        var rtf = '';
        log.split('\n')
            .filter(function (line) { return line.includes('endpoint detected'); })
            .forEach(function (line) {
            line.split(' ')
                .filter(function (value) { return value.includes(':') && value.includes('rtf'); })
                .map(function (line) { return line.split(':'); })
                .forEach(function (tuple) {
                if (tuple[0] === 'rtf')
                    rtf = tuple[1];
            });
        });
        if (rtf.length === 0)
            return undefined;
        return Number(rtf);
    };
    return ASRBenchmark;
}());
exports.ASRBenchmark = ASRBenchmark;
