import { AudioDevice } from '../utils/audiodevice';
import { Assistant } from '../models/assistant';
import { Credentials } from '../utils/config';
export declare const snipsRemoteAuthorizedKeysFilename = "~/.ssh/authorized_keys";
export declare const availableSnipsServices: string[];
export declare class SSHService {
    private ssh;
    credentials: Credentials;
    constructor();
    connect: () => Promise<any>;
    newConnection: (credentials: Credentials, password: string, onConnected: () => void, onSSHCopyId: () => void, onCredentialsSaved: () => void) => Promise<void>;
    private connectInternal;
    disconnect: () => Promise<void>;
    reboot(): Promise<void>;
    isAssistantInstalled: () => Promise<boolean>;
    getInstalledAssistant: () => Promise<Assistant | undefined>;
    getOSVersion: () => Promise<string>;
    installSnipsPlatformDemo: (listener: (output: string) => void) => Promise<string>;
    aptGet: (packageName: string, listener: (output: string) => void) => Promise<string>;
    isNetcatInstalled: () => Promise<boolean>;
    speakerTest: () => Promise<string>;
    stopSpeakerTest: () => Promise<string>;
    setHostname: (newHostname: string) => Promise<string>;
    static fetchBootHostname: (ips: string[]) => Promise<{
        ip: string;
        hostname: string;
        snipsHostname: string;
    }[]>;
    getCaptureLevel: (card?: number | undefined) => Promise<string>;
    setCaptureLevel: (card?: number | undefined, level?: number | undefined) => Promise<string>;
    getVolumeLevel: (card?: number | undefined) => Promise<string>;
    setVolumeLevel: (card?: number | undefined, level?: number | undefined) => Promise<string>;
    recordAudio: (name: string) => Promise<string>;
    stopRecording: () => Promise<string>;
    listCaptureDevices: () => Promise<AudioDevice[]>;
    listOuputDevices: () => Promise<AudioDevice[]>;
    launchSnipsWatch: (listener: (string: any) => void) => Promise<string>;
    journalctlLogs(services: string[] | undefined, listener: (string) => void): Promise<string>;
    snipsServicesSystemctlStatus: () => Promise<{
        name: string;
        active: boolean;
    }[]>;
    playAudio: (name: string) => Promise<string>;
    removeAudioFile: (name: string) => Promise<string>;
    setupAsoundConf: (outputDevice: AudioDevice, captureDevice: AudioDevice) => Promise<string>;
    copySnipsfile: (snipsfile: string) => Promise<any>;
    runSnipsmanagerInstall: (username: string, password: string, listener: (string: any) => void) => Promise<string>;
    installPip: (url: string, listener: (string: any) => void) => Promise<string>;
    runSnipsManager: (listener: (string: any) => void) => Promise<string>;
    updateSnips: (listener: (string: any) => void) => Promise<string>;
    installSnips: (listener: (string: any) => void) => Promise<string>;
    packagesVersion: () => Promise<{
        name: string;
        version: string;
    }[]>;
    updateWifi: (name: string, passphrase: string) => Promise<string>;
    statusWifi: () => Promise<string>;
    reconnectWifi: () => Promise<string>;
    disconnectWifi: () => Promise<string>;
    putFile: (localPath: string, remotePath: string) => Promise<string>;
    putDirectory: (localDirectory: string, remoteDirectory: string) => Promise<string>;
    removeFile: (remotePath: string) => Promise<string>;
    removeFolder: (remotePath: string) => Promise<string>;
    unzipFile: (filePath: string, destinationPath?: string) => Promise<string>;
    installAssistant: (assistantPath: string) => Promise<string>;
    dpkgCheckIfPackageIsInstalled: (packageName: string) => Promise<boolean>;
    installRespeakerMicHat(listener: (string) => void): Promise<string>;
    private sshCommandLog;
    playWAVtoASR: (remoteWAVPath: string) => Promise<string>;
    journalctlLog: (service: string, listener: (string: any) => void) => Promise<string>;
    editAudioServerService: () => Promise<string>;
    resetAudioServerService: () => Promise<string>;
    editASRService: (maxActive: number, minActive: number) => Promise<string>;
    resetASRService: () => Promise<string>;
    publishMQTTStartListening: () => Promise<string>;
    killNCServer: () => Promise<string>;
    sshCopyId: (hostname: string, username: string) => Promise<string>;
    private ssh_keygen_binPath();
    private sshKeygen;
    private addSnipsRSAKeyToSSHConfigIfNeeded();
}
