"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ora = require('ora')('');
var chalk_1 = require("chalk");
var Stream = (function () {
    function Stream(output) {
        this.tab = '';
        this.output = output;
    }
    Stream.prototype.loading = function (text) {
        ora.text = text;
        ora.start();
    };
    Stream.prototype.update = function (text) {
        ora.text = text;
    };
    Stream.prototype.stop = function () {
        ora.stopAndPersist();
    };
    Stream.prototype.stopAndClear = function () {
        ora.stop();
    };
    Stream.prototype.success = function (text, fullColor) {
        if (fullColor === void 0) { fullColor = false; }
        ora.succeed(fullColor && (text !== undefined) ? chalk_1.default.green(text) : text);
    };
    Stream.prototype.successNoCheck = function (text) {
        this.println(chalk_1.default.green(text));
    };
    Stream.prototype.done = function (appendDone) {
        if (appendDone === void 0) { appendDone = true; }
        if (ora.text) {
            ora.stop();
            if (appendDone) {
                this.println(ora.text + " done");
            }
            else {
                this.println("" + ora.text);
            }
        }
        else {
            ora.stopAndPersist();
        }
    };
    Stream.prototype.ok = function (text) {
        ora.stopAndPersist({ text: "[OK] " + text });
    };
    Stream.prototype.error = function (text, fullColor) {
        if (fullColor === void 0) { fullColor = true; }
        if (text === undefined) {
            ora.fail(undefined);
        }
        else {
            ora.fail(fullColor ? chalk_1.default.red(text) : text);
        }
    };
    Stream.prototype.errorPlain = function (text) {
        this.println(chalk_1.default.red("" + text));
    };
    Stream.prototype.hint = function (text) {
        this.output.write(chalk_1.default.blue("i") + (" " + text + " \n"));
    };
    Stream.prototype.println = function (text) {
        var _this = this;
        if (text === undefined) {
            this.output.write('\n');
            return;
        }
        var indented = text.split('\n').map(function (l) { return _this.tab + l; }).join('\n');
        this.output.write(indented + '\n');
    };
    Stream.prototype.print = function (text) {
        var _this = this;
        var indented = text.split('\n').map(function (l) { return _this.tab + l; }).join('\n');
        this.output.write(indented);
    };
    Stream.prototype.log = function (tag, text) {
        this.println("[" + tag + "] " + text);
    };
    return Stream;
}());
exports.Stream = Stream;
