import * as yargs from 'yargs';
import { cli } from '../cli';
import { SSHService } from '../session/ssh';
import chalk from 'chalk';

export const command = 'install <assistant_path>';
export const desc = 'Install an assistant to the device';

export const handler = async (argv: yargs.Argv) => {
    const assistantPath: string = JSON.parse(JSON.stringify(argv)).assistant_path;

    const ssh = new SSHService();
    await ssh.connect()
    .catch((e) => {
        cli.stream.error(e);
        process.exit();
    });

    cli.stream.loading(`Deploying assistant to ${ssh.credentials.hostname}...`);
    await ssh.installAssistant(assistantPath)
    .catch((e) => {
        cli.stream.error('Unable to install the assistant:' + e.message);
        ssh.disconnect();
        process.exit();
    });
    cli.stream.success();

    const assistant = await ssh.getInstalledAssistant();
    if (assistant !== undefined) {
        cli.stream.success(`Snips assistant is now running. Say ${chalk.blue(assistant.hotword)} to start!`);
    } else {
        cli.stream.success(`Snips assistant is now running`);
    }

    ssh.disconnect();
    process.exit();
};
