import chalk from 'chalk';
import * as yargs from 'yargs';
import { cli } from '../../cli';
import { SSHService } from '../../session/ssh';

export const command = 'microphone';
export const desc = 'Test a microphone';

export const handler = async (argv: yargs.Argv) => {
    const ssh = new SSHService();
    await ssh.connect()
    .catch((e) => {
        cli.stream.error(e);
        process.exit();
    });

    cli.stream.hint('Plug in your microphone and turn on your speaker');

    try {
        const recordingFile = 'sam_recording';
        cli.prompt.waitForKeypress('Say something in the microphone, then press a key to continue...', async () => {
            cli.stream.success();
            await ssh.stopRecording().catch((e) => {});
        });
        await ssh.recordAudio(recordingFile).catch((e) => {});

        // await ssh.stopRecording().catch((e) => {});
        cli.stream.loading('Playing the recording');
        await ssh.playAudio(recordingFile).catch((e) => {});
        cli.stream.success();
    } catch (e) {
        cli.stream.stopAndClear();
        cli.stream.error('Error testing the microphone: ' + e.message);
        ssh.disconnect();
        process.exit();
    }
    cli.stream.hint(`Didn't hear anything? Check the recording level of your microphone and the volume of the speaker.`);
    cli.stream.hint(`Alternatively you can run ` + chalk.blue('sam setup audio') + ' to setup your microphone');
    ssh.disconnect();
    process.exit();
};
