import chalk from 'chalk';
import * as yargs from 'yargs';
import { cli } from '../../cli';
import { SSHService } from '../../session/ssh';

export const command = 'speaker';
export const desc = 'Test your device speaker';

export const handler = async (argv: yargs.Argv) => {
    cli.stream.println('Testing speaker');
    const ssh = new SSHService();
    await ssh.connect()
    .catch((e) => {
        cli.stream.error(e);
        process.exit();
    });

    try {
        ssh.speakerTest().catch(e => cli.stream.error(e));
        await cli.prompt.waitForKeypress('Playing sound on your speaker. Press Enter to stop...', () => {
            cli.stream.success();
        }).catch((e) => {});
        ssh.stopSpeakerTest().catch((e) => {});

    } catch (e) {
        cli.stream.stop();
        cli.stream.error('Error testing the speaker: ' + e.message);
        ssh.disconnect();
        process.exit();
    }
    cli.stream.done();
    cli.stream.hint(`Didn't hear anything ? Run : ` + chalk.blue(`sam setup audio`));
    ssh.disconnect();
    process.exit();
};
